@@ -24,7 +24,9 @@ SUPPORTED_API_EXTENSIONS = [
     'router',
     'l3_agent_scheduler',
     'subnet_allocation',
-    'dvr',
     'port-security',
-    'allowed-address-pairs'
+    'allowed-address-pairs',
+    'net-mtu',
+    'default-subnetpools',
+    'extraroute',
 ]
