@@ -17,32 +17,38 @@ import socket
 import sys
 import time
 
-import eventlet
-
-from oslo_config import cfg
-from oslo_log import log
-from oslo_utils import importutils
-
 from neutron.agent.common import config
 from neutron.common import config as common_config
+from oslo_config import cfg
+from oslo_log import log
+from oslo_serialization import jsonutils
+from ryu.base import app_manager
+from ryu import cfg as ryu_cfg
+from ryu.ofproto import ofproto_common
 
-from dragonflow._i18n import _LI, _LW
+from dragonflow._i18n import _LI, _LW, _
 from dragonflow.common import common_params
 from dragonflow.common import constants
-from dragonflow.controller.ryu_base_app import RyuDFAdapter
-from dragonflow.controller.topology import Topology
+from dragonflow.common import utils as df_utils
+from dragonflow.controller import ryu_base_app
+from dragonflow.controller import topology
 from dragonflow.db import api_nb
 from dragonflow.db import db_store
 from dragonflow.db.drivers import ovsdb_vswitch_impl
 
-from ryu.base.app_manager import AppManager
+
+df_ryu_opts = [
+    cfg.IPOpt('of_listen_address', default='127.0.0.1',
+              help=_("Address to listen on for OpenFlow connections.")),
+    cfg.PortOpt('of_listen_port', default=ofproto_common.OFP_TCP_PORT,
+                help=_("Port to listen on for OpenFlow connections."))
+]
 
 config.setup_logging()
 LOG = log.getLogger("dragonflow.controller.df_local_controller")
 
-eventlet.monkey_patch()
-
 cfg.CONF.register_opts(common_params.df_opts, 'df')
+cfg.CONF.register_opts(df_ryu_opts, 'df_ryu')
 
 
 class DfLocalController(object):
@@ -54,45 +60,50 @@ class DfLocalController(object):
         self.ip = cfg.CONF.df.local_ip
         self.tunnel_type = cfg.CONF.df.tunnel_type
         self.sync_finished = False
-        nb_driver_class = importutils.import_class(cfg.CONF.df.nb_db_class)
+        nb_driver = df_utils.load_driver(
+            cfg.CONF.df.nb_db_class,
+            df_utils.DF_NB_DB_DRIVER_NAMESPACE)
         self.nb_api = api_nb.NbApi(
-            nb_driver_class(),
+            nb_driver,
             use_pubsub=cfg.CONF.df.enable_df_pub_sub)
-        self.vswitch_api = ovsdb_vswitch_impl.OvsdbSwitchApi(
-            self.ip, self.nb_api)
+        self.vswitch_api = ovsdb_vswitch_impl.OvsdbSwitchApi(self.ip)
         kwargs = dict(
             nb_api=self.nb_api,
             vswitch_api=self.vswitch_api,
             db_store=self.db_store
         )
-        app_mgr = AppManager.get_instance()
-        self.open_flow_app = app_mgr.instantiate(RyuDFAdapter, **kwargs)
+        app_mgr = app_manager.AppManager.get_instance()
+        self.open_flow_app = app_mgr.instantiate(ryu_base_app.RyuDFAdapter,
+                                                 **kwargs)
 
         self.topology = None
         self.enable_selective_topo_dist = \
             cfg.CONF.df.enable_selective_topology_distribution
+        self.remote_chassis_lport_map = {}
+        self.integration_bridge = cfg.CONF.df.integration_bridge
 
     def run(self):
         self.nb_api.initialize(db_ip=cfg.CONF.df.remote_db_ip,
                                db_port=cfg.CONF.df.remote_db_port)
-        self.vswitch_api.initialize()
-        self.topology = Topology(self, self.enable_selective_topo_dist)
+        self.vswitch_api.initialize(self.nb_api)
+        self.topology = topology.Topology(self,
+                                          self.enable_selective_topo_dist)
 
         self.vswitch_api.sync()
         # both set_controller and del_controller will delete flows.
         # for reliability, here we should check if controller is set for OVS,
         # if yes, don't set controller and don't delete controller.
         # if no, set controller
-        # TODO(heshan) port should be configured in cfg file
-        targets = 'tcp:' + self.ip + ':6633'
+        targets = ('tcp:' + cfg.CONF.df_ryu.of_listen_address + ':' +
+            str(cfg.CONF.df_ryu.of_listen_port))
         is_controller_set = self.vswitch_api.check_controller(targets)
         if not is_controller_set:
-            self.vswitch_api.set_controllers('br-int', [targets]).execute()
+            self.vswitch_api.set_controller(self.integration_bridge, [targets])
         is_fail_mode_set = self.vswitch_api.check_controller_fail_mode(
             'secure')
         if not is_fail_mode_set:
             self.vswitch_api.set_controller_fail_mode(
-                'br-int', 'secure').execute()
+                self.integration_bridge, 'secure')
         self.open_flow_app.start()
         self.db_sync_loop()
 
@@ -153,19 +164,25 @@ class DfLocalController(object):
         # Create tunnel port to this chassis
         LOG.info(_LI("Adding tunnel to remote chassis = %s") %
                  chassis.__str__())
-        self.vswitch_api.add_tunnel_port(chassis).execute()
+        self.vswitch_api.add_tunnel_port(chassis)
 
     def chassis_deleted(self, chassis_id):
         LOG.info(_LI("Deleting tunnel to remote chassis = %s") % chassis_id)
         tunnel_ports = self.vswitch_api.get_tunnel_ports()
         for port in tunnel_ports:
             if port.get_chassis_id() == chassis_id:
-                self.vswitch_api.delete_port(port).execute()
+                self.vswitch_api.delete_port(port)
                 return
 
     def read_switches(self):
+        lswitches_to_remove = self.db_store.get_lswitch_keys()
         for lswitch in self.nb_api.get_all_logical_switches():
             self.logical_switch_updated(lswitch)
+            if lswitch.get_id() in lswitches_to_remove:
+                lswitches_to_remove.remove(lswitch.get_id())
+
+        for lswitch_to_remove in lswitches_to_remove:
+            self.logical_switch_deleted(lswitch_to_remove)
 
     def logical_switch_updated(self, lswitch):
         old_lswitch = self.db_store.get_lswitch(lswitch.get_id())
@@ -192,23 +209,33 @@ class DfLocalController(object):
         self.db_store.del_lswitch(lswitch_id)
         self.db_store.del_network_id(lswitch_id)
 
-    def _logical_port_process(self, lport, original_lport=None):
-        if lport.get_chassis() is None or (
-                    lport.get_chassis() == constants.DRAGONFLOW_VIRTUAL_PORT):
-            return
+    def _is_physical_chassis(self, chassis):
+        if not chassis or chassis == constants.DRAGONFLOW_VIRTUAL_PORT:
+            return False
+        return True
 
-        chassis_to_ofport, lport_to_ofport = (
-            self.vswitch_api.get_local_ports_to_ofport_mapping())
-        network = self.get_network_id(
+    def _logical_port_process(self, lport, original_lport=None):
+        chassis = lport.get_chassis()
+        local_network_id = self.get_network_id(
             lport.get_lswitch_id(),
         )
-        lport.set_external_value('local_network_id', network)
+        lswitch = self.db_store.get_lswitch(lport.get_lswitch_id())
+        if lswitch is not None:
+            network_type = lswitch.get_network_type()
+            segment_id = lswitch.get_segment_id()
 
-        if lport.get_chassis() == self.chassis_name:
+            lport.set_external_value('network_type', network_type)
+            if segment_id is not None:
+                lport.set_external_value('segmentation_id',
+                                     int(segment_id))
+        lport.set_external_value('local_network_id', local_network_id)
+
+        if chassis == self.chassis_name:
             lport.set_external_value('is_local', True)
-            ofport = lport_to_ofport.get(lport.get_id(), 0)
-            if ofport != 0:
+            ofport = self.vswitch_api.get_port_ofport_by_id(lport.get_id())
+            if ofport:
                 lport.set_external_value('ofport', ofport)
+                self.db_store.set_port(lport.get_id(), lport, True)
                 if original_lport is None:
                     LOG.info(_LI("Adding new local logical port = %s") %
                              str(lport))
@@ -223,12 +250,12 @@ class DfLocalController(object):
             else:
                 LOG.info(_LI("Local logical port %s was not created yet") %
                          str(lport))
-            self.db_store.set_port(lport.get_id(), lport, True)
         else:
             lport.set_external_value('is_local', False)
-            ofport = chassis_to_ofport.get(lport.get_chassis(), 0)
-            if ofport != 0:
+            ofport = self.vswitch_api.get_chassis_ofport(chassis)
+            if ofport:
                 lport.set_external_value('ofport', ofport)
+                self.db_store.set_port(lport.get_id(), lport, False)
                 if original_lport is None:
                     LOG.info(_LI("Adding new remote logical port = %s") %
                              str(lport))
@@ -246,13 +273,57 @@ class DfLocalController(object):
                 # if this should never happen raise an exception
                 LOG.warning(_LW("No tunnel for remote logical port %s") %
                             str(lport))
-            self.db_store.set_port(lport.get_id(), lport, False)
+
+    def _add_remote_port_on_chassis(self, lport):
+        chassis = lport.get_chassis()
+        chassis_lports = self.remote_chassis_lport_map.get(chassis)
+        if not chassis_lports:
+            chassis_value = {'id': chassis, 'ip': chassis,
+                             'tunnel_type': self.tunnel_type}
+            chassis_inst = api_nb.Chassis(jsonutils.dumps(chassis_value))
+            self.chassis_created(chassis_inst)
+            self.remote_chassis_lport_map[chassis] = {lport.get_id()}
+        else:
+            chassis_lports.add(lport.get_id())
+
+    def _delete_remote_port_from_chassis(self, lport):
+        chassis = lport.get_chassis()
+        chassis_lports = self.remote_chassis_lport_map.get(chassis)
+        if chassis_lports is None:
+            return
+        chassis_lports.remove(lport.get_id())
+        if len(chassis_lports) == 0:
+            self.chassis_deleted(chassis)
+            del self.remote_chassis_lport_map[chassis]
 
     def logical_port_created(self, lport):
+        chassis = lport.get_chassis()
+        if not self._is_physical_chassis(chassis):
+            LOG.debug(("Port %s has not been bound or it is a vPort") %
+                      lport.get_id())
+            return
+        if lport.get_remote_vtep():
+            self._add_remote_port_on_chassis(lport)
         self._logical_port_process(lport)
 
     def logical_port_updated(self, lport):
+        chassis = lport.get_chassis()
+        if not self._is_physical_chassis(chassis):
+            LOG.debug(("Port %s has not been bound or it is a vPort") %
+                      lport.get_id())
+            return
         original_lport = self.db_store.get_port(lport.get_id())
+        if not original_lport:
+            if lport.get_remote_vtep():
+                self._add_remote_port_on_chassis(lport)
+        else:
+            original_chassis = original_lport.get_chassis()
+            if original_chassis != chassis:
+                if original_lport.get_remote_vtep():
+                    self._delete_remote_port_from_chassis(original_lport)
+
+                if lport.get_remote_vtep():
+                    self._add_remote_port_on_chassis(lport)
         self._logical_port_process(lport, original_lport)
 
     def logical_port_deleted(self, lport_id):
@@ -272,6 +343,12 @@ class DfLocalController(object):
                 self.open_flow_app.notify_remove_remote_port(lport)
             self.db_store.delete_port(lport.get_id(), False)
 
+        if lport.get_remote_vtep():
+            self._delete_remote_port_from_chassis(lport)
+
+    def bridge_port_updated(self, lport):
+        self.open_flow_app.notify_update_bridge_port(lport)
+
     def router_updated(self, lrouter):
         old_lrouter = self.db_store.get_router(lrouter.get_id())
         if old_lrouter is None:
@@ -280,6 +357,7 @@ class DfLocalController(object):
             self._add_new_lrouter(lrouter)
             return
         self._update_router_interfaces(old_lrouter, lrouter)
+        self._update_router_attributes(old_lrouter, lrouter)
         self.db_store.update_router(lrouter.get_id(), lrouter)
 
     def router_deleted(self, lrouter_id):
@@ -332,7 +410,7 @@ class DfLocalController(object):
 
         # Iterate all tunnel ports that needs to be deleted
         for port in tunnel_ports.values():
-            self.vswitch_api.delete_port(port).execute()
+            self.vswitch_api.delete_port(port)
 
     def port_mappings(self):
         ports_to_remove = self.db_store.get_port_keys()
@@ -358,8 +436,35 @@ class DfLocalController(object):
             return self.next_network_id
 
     def read_routers(self):
+        routers_to_remove = self.db_store.get_router_keys()
         for lrouter in self.nb_api.get_routers():
             self.router_updated(lrouter)
+            if lrouter.get_id() in routers_to_remove:
+                routers_to_remove.remove(lrouter.get_id())
+
+        for router_to_remove in routers_to_remove:
+            self.router_deleted(router_to_remove)
+
+    def _update_router_attributes(self, old_router, new_router):
+        old_routes = old_router.get_routes()
+        new_routes = new_router.get_routes()
+        for new_route in new_routes:
+            if new_route not in old_routes:
+                self._add_router_route(new_router, new_route)
+            else:
+                old_routes.remove(new_route)
+        for old_route in old_routes:
+            self._delete_router_route(new_router, old_route)
+
+    def _add_router_route(self, router, route):
+        LOG.info(_LI("Adding new logical router route = %s"), route)
+        self.open_flow_app.notify_add_router_route(
+            router, route)
+
+    def _delete_router_route(self, router, route):
+        LOG.info(_LI("Removing logical router route = %s"), route)
+        self.open_flow_app.notify_remove_router_route(
+            router, route)
 
     def _update_router_interfaces(self, old_router, new_router):
         new_router_ports = new_router.get_ports()
@@ -394,6 +499,9 @@ class DfLocalController(object):
     def _add_new_lrouter(self, lrouter):
         for new_port in lrouter.get_ports():
             self._add_new_router_port(lrouter, new_port)
+        routes = lrouter.get_routes()
+        for route in routes:
+            self._add_router_route(lrouter, route)
         self.db_store.update_router(lrouter.get_id(), lrouter)
 
     def read_security_groups(self):
@@ -442,20 +550,21 @@ class DfLocalController(object):
                  secgroup, secgroup_rule)
 
     def read_floatingip(self):
+        fips_to_remove = self.db_store.get_floatingip_keys()
         for floatingip in self.nb_api.get_floatingips():
             self.floatingip_updated(floatingip)
+            if floatingip.get_id() in fips_to_remove:
+                fips_to_remove.remove(floatingip.get_id())
+
+        for fip_to_remove in fips_to_remove:
+            self.floatingip_deleted(fip_to_remove)
 
     def floatingip_updated(self, floatingip):
         # check whether this floatingip is associated with a lport or not
         if floatingip.get_lport_id():
             if self.db_store.get_local_port(floatingip.get_lport_id()) is None:
                 return
-        if floatingip.get_lrouter_id():
-            lrouter = self.db_store.get_router(floatingip.get_lrouter_id())
-            # Currently, to implement DNAT for DVR on compute node only
-            # if distributed is False, DNAT is done on centralized vrouter
-            if not lrouter.is_distributed():
-                return
+
         old_floatingip = self.db_store.get_floatingip(floatingip.get_id())
         if old_floatingip is None:
             # The new floatingip should be associated with a lport
@@ -490,13 +599,13 @@ class DfLocalController(object):
     def _associate_floatingip(self, floatingip):
         self.db_store.update_floatingip(floatingip.get_id(), floatingip)
         self.open_flow_app.notify_associate_floatingip(floatingip)
-        LOG.info(_LI("Floatingip is assoicated with port. Floatingip = %s") %
+        LOG.info(_LI("Floatingip is associated with port. Floatingip = %s") %
                  str(floatingip))
 
     def _disassociate_floatingip(self, floatingip):
         self.db_store.delete_floatingip(floatingip.get_id())
         self.open_flow_app.notify_disassociate_floatingip(floatingip)
-        LOG.info(_LI("Floatingip is disassoicated from port."
+        LOG.info(_LI("Floatingip is disassociated from port."
                  " Floatingip = %s") % str(floatingip))
 
     def _update_floatingip(self, old_floatingip, new_floatingip):
@@ -506,10 +615,12 @@ class DfLocalController(object):
                 self._associate_floatingip(new_floatingip)
 
     def ovs_port_updated(self, ovs_port):
+        self.open_flow_app.notify_ovs_port_updated(ovs_port)
         self.topology.ovs_port_updated(ovs_port)
 
-    def ovs_port_deleted(self, ovs_port_id):
-        self.topology.ovs_port_deleted(ovs_port_id)
+    def ovs_port_deleted(self, ovs_port):
+        self.open_flow_app.notify_ovs_port_deleted(ovs_port)
+        self.topology.ovs_port_deleted(ovs_port.get_id())
 
     def ovs_sync_finished(self):
         self.open_flow_app.notify_ovs_sync_finished()
@@ -530,14 +641,18 @@ class DfLocalController(object):
         return self.chassis_name
 
 
+def init_ryu_config():
+    ryu_cfg.CONF(project='ryu', args=[])
+    ryu_cfg.CONF.ofp_listen_host = cfg.CONF.df_ryu.of_listen_address
+    ryu_cfg.CONF.ofp_tcp_listen_port = cfg.CONF.df_ryu.of_listen_port
+
+
 # Run this application like this:
 # python df_local_controller.py <chassis_unique_name>
 # <local ip address> <southbound_db_ip_address>
 def main():
     chassis_name = socket.gethostname()
     common_config.init(sys.argv[1:])
+    init_ryu_config()
     controller = DfLocalController(chassis_name)
     controller.run()
-
-if __name__ == "__main__":
-    main()
