@@ -14,17 +14,14 @@
 #    under the License.
 
 import netaddr
-
-from neutron.common import constants as common_const
-
+from neutron_lib import constants as common_const
+from oslo_config import cfg
 from ryu.lib.mac import haddr_to_bin
 
 from dragonflow._i18n import _
-from dragonflow.controller.common.arp_responder import ArpResponder
+from dragonflow.controller.common import arp_responder
 from dragonflow.controller.common import constants as const
-from dragonflow.controller.df_base_app import DFlowApp
-
-from oslo_config import cfg
+from dragonflow.controller import df_base_app
 
 DF_L2_APP_OPTS = [
     cfg.BoolOpt(
@@ -41,7 +38,7 @@ DF_L2_APP_OPTS = [
 OF_IN_PORT = 0xfff8
 
 
-class L2App(DFlowApp):
+class L2App(df_base_app.DFlowApp):
 
     def __init__(self, *args, **kwargs):
         super(L2App, self).__init__(*args, **kwargs)
@@ -84,34 +81,76 @@ class L2App(DFlowApp):
                                   const.PRIORITY_DEFAULT,
                                   const.INGRESS_DISPATCH_TABLE)
 
-        self._install_flows_on_switch_up()
+        # Clear local networks cache so the multicast/broadcast flows
+        # are installed correctly
+        self.local_networks.clear()
 
     def _add_arp_responder(self, lport):
         if not self.is_install_arp_responder:
             return
+        if lport.get_device_owner() == common_const.DEVICE_OWNER_ROUTER_INTF:
+            return
         ip = lport.get_ip()
         if netaddr.IPAddress(ip).version != 4:
             return
         network_id = lport.get_external_value('local_network_id')
         mac = lport.get_mac()
-        ArpResponder(self.get_datapath(), network_id, ip, mac).add()
+        arp_responder.ArpResponder(self.get_datapath(),
+                                   network_id, ip, mac).add()
 
     def _remove_arp_responder(self, lport):
         if not self.is_install_arp_responder:
             return
+        if lport.get_device_owner() == common_const.DEVICE_OWNER_ROUTER_INTF:
+            return
         ip = lport.get_ip()
         if netaddr.IPAddress(ip).version != 4:
             return
         network_id = lport.get_external_value('local_network_id')
-        ArpResponder(self.get_datapath(), network_id, ip).remove()
+        arp_responder.ArpResponder(self.get_datapath(),
+                                   network_id, ip).remove()
+
+    def _add_dst_classifier_flow_for_port(self, network_id, mac, tunnel_key):
+        parser = self.get_datapath().ofproto_parser
+        ofproto = self.get_datapath().ofproto
+        match = parser.OFPMatch()
+        match.set_metadata(network_id)
+        match.set_dl_dst(haddr_to_bin(mac))
+        actions = []
+        actions.append(parser.OFPActionSetField(reg7=tunnel_key))
+        action_inst = self.get_datapath().ofproto_parser\
+            .OFPInstructionActions(ofproto.OFPIT_APPLY_ACTIONS, actions)
+        goto_inst = parser.OFPInstructionGotoTable(const.EGRESS_TABLE)
+        inst = [action_inst, goto_inst]
+        self.mod_flow(
+            self.get_datapath(),
+            inst=inst,
+            table_id=const.L2_LOOKUP_TABLE,
+            priority=const.PRIORITY_MEDIUM,
+            match=match)
+
+    def _delete_dst_classifier_flow_for_port(self, network_id, mac):
+        parser = self.get_datapath().ofproto_parser
+        ofproto = self.get_datapath().ofproto
+        match = parser.OFPMatch()
+        match.set_metadata(network_id)
+        match.set_dl_dst(haddr_to_bin(mac))
+        self.mod_flow(
+            datapath=self.get_datapath(),
+            table_id=const.L2_LOOKUP_TABLE,
+            command=ofproto.OFPFC_DELETE,
+            priority=const.PRIORITY_MEDIUM,
+            out_port=ofproto.OFPP_ANY,
+            out_group=ofproto.OFPG_ANY,
+            match=match)
 
     def remove_local_port(self, lport):
-
         lport_id = lport.get_id()
         mac = lport.get_mac()
         network_id = lport.get_external_value('local_network_id')
         ofport = lport.get_external_value('ofport')
         tunnel_key = lport.get_tunnel_key()
+        device_owner = lport.get_device_owner()
 
         parser = self.get_datapath().ofproto_parser
         ofproto = self.get_datapath().ofproto
@@ -152,17 +191,8 @@ class L2App(DFlowApp):
         self.get_datapath().send_msg(msg)
 
         # Remove destination classifier for port
-        match = parser.OFPMatch()
-        match.set_metadata(network_id)
-        match.set_dl_dst(haddr_to_bin(mac))
-        self.mod_flow(
-            datapath=self.get_datapath(),
-            table_id=const.L2_LOOKUP_TABLE,
-            command=ofproto.OFPFC_DELETE,
-            priority=const.PRIORITY_MEDIUM,
-            out_port=ofproto.OFPP_ANY,
-            out_group=ofproto.OFPG_ANY,
-            match=match)
+        if device_owner != common_const.DEVICE_OWNER_ROUTER_INTF:
+            self._delete_dst_classifier_flow_for_port(network_id, mac)
 
         # Remove egress classifier for port
         match = parser.OFPMatch(reg7=tunnel_key)
@@ -174,9 +204,7 @@ class L2App(DFlowApp):
             out_port=ofproto.OFPP_ANY,
             out_group=ofproto.OFPG_ANY,
             match=match)
-
         self._del_multicast_broadcast_handling_for_port(network_id, lport_id)
-
         self._remove_arp_responder(lport)
 
     def remove_remote_port(self, lport):
@@ -185,22 +213,14 @@ class L2App(DFlowApp):
         mac = lport.get_mac()
         network_id = lport.get_external_value('local_network_id')
         tunnel_key = lport.get_tunnel_key()
+        device_owner = lport.get_device_owner()
 
         parser = self.get_datapath().ofproto_parser
         ofproto = self.get_datapath().ofproto
 
         # Remove destination classifier for port
-        match = parser.OFPMatch()
-        match.set_metadata(network_id)
-        match.set_dl_dst(haddr_to_bin(mac))
-        self.mod_flow(
-            datapath=self.get_datapath(),
-            table_id=const.L2_LOOKUP_TABLE,
-            command=ofproto.OFPFC_DELETE,
-            priority=const.PRIORITY_MEDIUM,
-            out_port=ofproto.OFPP_ANY,
-            out_group=ofproto.OFPG_ANY,
-            match=match)
+        if device_owner != common_const.DEVICE_OWNER_ROUTER_INTF:
+            self._delete_dst_classifier_flow_for_port(network_id, mac)
 
         # Remove egress classifier for port
         match = parser.OFPMatch(reg7=tunnel_key)
@@ -212,13 +232,10 @@ class L2App(DFlowApp):
             out_port=ofproto.OFPP_ANY,
             out_group=ofproto.OFPG_ANY,
             match=match)
-
         self._del_multicast_broadcast_handling_for_port(network_id, lport_id)
-
         self._remove_arp_responder(lport)
 
     def add_local_port(self, lport):
-
         if self.get_datapath() is None:
             return
 
@@ -227,6 +244,7 @@ class L2App(DFlowApp):
         network_id = lport.get_external_value('local_network_id')
         ofport = lport.get_external_value('ofport')
         tunnel_key = lport.get_tunnel_key()
+        device_owner = lport.get_device_owner()
 
         parser = self.get_datapath().ofproto_parser
         ofproto = self.get_datapath().ofproto
@@ -280,30 +298,10 @@ class L2App(DFlowApp):
             priority=const.PRIORITY_MEDIUM,
             match=match)
 
-        # Destination classifier for port
-        priority = const.PRIORITY_MEDIUM
-        goto_table = const.EGRESS_TABLE
-
-        # Router MAC's go to L3 table and have higher priority
-        if lport.get_device_owner() == common_const.DEVICE_OWNER_ROUTER_INTF:
-            priority = const.PRIORITY_HIGH
-            goto_table = const.L3_LOOKUP_TABLE
-
-        match = parser.OFPMatch()
-        match.set_metadata(network_id)
-        match.set_dl_dst(haddr_to_bin(mac))
-        actions = []
-        actions.append(parser.OFPActionSetField(reg7=tunnel_key))
-        action_inst = self.get_datapath().ofproto_parser.OFPInstructionActions(
-            ofproto.OFPIT_APPLY_ACTIONS, actions)
-        goto_inst = parser.OFPInstructionGotoTable(goto_table)
-        inst = [action_inst, goto_inst]
-        self.mod_flow(
-            self.get_datapath(),
-            inst=inst,
-            table_id=const.L2_LOOKUP_TABLE,
-            priority=priority,
-            match=match)
+        # Router MAC's go to L3 table will be handled by l3_app
+        if device_owner != common_const.DEVICE_OWNER_ROUTER_INTF:
+            self._add_dst_classifier_flow_for_port(
+                    network_id, mac, tunnel_key)
 
         # Egress classifier for port
         match = parser.OFPMatch(reg7=tunnel_key)
@@ -420,7 +418,6 @@ class L2App(DFlowApp):
             match=match)
 
     def add_remote_port(self, lport):
-
         if self.get_datapath() is None:
             return
 
@@ -429,34 +426,15 @@ class L2App(DFlowApp):
         network_id = lport.get_external_value('local_network_id')
         ofport = lport.get_external_value('ofport')
         tunnel_key = lport.get_tunnel_key()
+        device_owner = lport.get_device_owner()
 
         parser = self.get_datapath().ofproto_parser
         ofproto = self.get_datapath().ofproto
 
-        # Destination classifier for port
-        priority = const.PRIORITY_MEDIUM
-        goto_table = const.EGRESS_TABLE
-
-        # Router MAC's go to L3 table and have higher priority
-        if lport.get_device_owner() == common_const.DEVICE_OWNER_ROUTER_INTF:
-            priority = const.PRIORITY_HIGH
-            goto_table = const.L3_LOOKUP_TABLE
-
-        match = parser.OFPMatch()
-        match.set_metadata(network_id)
-        match.set_dl_dst(haddr_to_bin(mac))
-        actions = []
-        actions.append(parser.OFPActionSetField(reg7=tunnel_key))
-        action_inst = self.get_datapath().ofproto_parser.OFPInstructionActions(
-            ofproto.OFPIT_APPLY_ACTIONS, actions)
-        goto_inst = parser.OFPInstructionGotoTable(goto_table)
-        inst = [action_inst, goto_inst]
-        self.mod_flow(
-            self.get_datapath(),
-            inst=inst,
-            table_id=const.L2_LOOKUP_TABLE,
-            priority=priority,
-            match=match)
+        # Router MAC's go to L3 table will be handled by l3_app
+        if device_owner != common_const.DEVICE_OWNER_ROUTER_INTF:
+            self._add_dst_classifier_flow_for_port(
+                    network_id, mac, tunnel_key)
 
         # Egress classifier for port
         match = parser.OFPMatch(reg7=tunnel_key)
@@ -472,18 +450,6 @@ class L2App(DFlowApp):
             table_id=const.EGRESS_TABLE,
             priority=const.PRIORITY_MEDIUM,
             match=match)
-
         self._add_multicast_broadcast_handling_for_port(network_id, lport_id,
                                                         tunnel_key)
-
         self._add_arp_responder(lport)
-
-    def _install_flows_on_switch_up(self):
-        # Clear local networks cache so the multicast/broadcast flows
-        # are installed correctly
-        self.local_networks.clear()
-        for port in self.db_store.get_ports():
-            if port.get_external_value('is_local'):
-                self.add_local_port(port)
-            else:
-                self.add_remote_port(port)
