@@ -13,13 +13,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 import netaddr
+from neutron.agent.common import config
+from neutron_lib import constants as n_const
+from oslo_log import log
+from ryu.ofproto import ether
 
 from dragonflow._i18n import _LI, _LE
 from dragonflow.controller.common import constants as const
-from dragonflow.controller.df_base_app import DFlowApp
-from neutron.agent.common import config
-from oslo_log import log
-from ryu.ofproto import ether
+from dragonflow.controller import df_base_app
 
 
 config.setup_logging()
@@ -31,7 +32,7 @@ COOKIE_FULLMASK = 0xffffffffffffffff
 SG_PRIORITY_OFFSET = 2
 
 
-class SGApp(DFlowApp):
+class SGApp(df_base_app.DFlowApp):
 
     def __init__(self, *args, **kwargs):
         super(SGApp, self).__init__(*args, **kwargs)
@@ -49,7 +50,6 @@ class SGApp(DFlowApp):
 
     @staticmethod
     def _split_range(range_start, range_end, full_mask):
-
         bit_flag = 1
         last_temp_start = range_start
         last_temp_end = last_temp_start
@@ -231,7 +231,6 @@ class SGApp(DFlowApp):
 
     @staticmethod
     def _get_rule_flows_match_except_net_addresses(secgroup_rule):
-
         protocol = secgroup_rule.get_protocol()
         port_range_max = secgroup_rule.get_port_range_max()
         port_range_min = secgroup_rule.get_port_range_min()
@@ -241,15 +240,15 @@ class SGApp(DFlowApp):
         dl_type_match = {}
         protocol_match = {}
         port_match_list = [{}]
-        if ethertype == 'IPv4':
+        if ethertype == n_const.IPv4:
             dl_type_match["eth_type"] = ether.ETH_TYPE_IP
             if protocol is not None:
-                if protocol == 'icmp':
-                    protocol = 1
-                elif protocol == 'tcp':
-                    protocol = 6
-                elif protocol == 'udp':
-                    protocol = 17
+                if protocol == n_const.PROTO_NAME_ICMP:
+                    protocol = n_const.PROTO_NUM_ICMP
+                elif protocol == n_const.PROTO_NAME_TCP:
+                    protocol = n_const.PROTO_NUM_TCP
+                elif protocol == n_const.PROTO_NAME_UDP:
+                    protocol = n_const.PROTO_NUM_UDP
                 else:
                     protocol = int(protocol)
 
@@ -277,11 +276,11 @@ class SGApp(DFlowApp):
                                 {port_match_name:
                                  SGApp._get_port_range_match(port_item)}
                             )
-        elif ethertype == 'IPv6':
+        elif ethertype == n_const.IPv6:
             # not support yet
             dl_type_match["eth_type"] = ether.ETH_TYPE_IPV6
         else:
-            LOG.error(_LE("wrong ethernet type"))
+            LOG.error(_LE("wrong Ethernet type"))
 
         for port_match in port_match_list:
             parameters_merge = dl_type_match.copy()
@@ -556,14 +555,14 @@ class SGApp(DFlowApp):
         match_list = \
             SGApp._get_rule_flows_match_except_net_addresses(secgroup_rule)
 
-        if secgroup_rule.get_ethertype() == 'IPv4':
+        if secgroup_rule.get_ethertype() == n_const.IPv4:
             if secgroup_rule.get_direction() == 'ingress':
                 table_id = const.INGRESS_SECURITY_GROUP_TABLE
                 ipv4_match_item = "ipv4_src"
             else:
                 table_id = const.EGRESS_SECURITY_GROUP_TABLE
                 ipv4_match_item = "ipv4_dst"
-        elif secgroup_rule.get_ethertype() == 'IPv6':
+        elif secgroup_rule.get_ethertype() == n_const.IPv6:
             # not support yet
             LOG.info(_LI("IPv6 rules are not supported yet"))
             return
@@ -643,7 +642,7 @@ class SGApp(DFlowApp):
             ofproto.OFPIT_APPLY_ACTIONS, actions)
         inst = [action_inst]
 
-        if ethertype == 'IPv4':
+        if ethertype == n_const.IPv4:
             addresses_list = [{}]
             if remote_group_id is not None:
                 aggregate_addresses_range = \
@@ -676,7 +675,7 @@ class SGApp(DFlowApp):
                         table_id=table_id,
                         priority=priority,
                         match=match)
-        elif ethertype == 'IPv6':
+        elif ethertype == n_const.IPv6:
             # not support yet
             LOG.info(_LI("IPv6 rules are not supported yet"))
         else:
@@ -716,6 +715,7 @@ class SGApp(DFlowApp):
             goto_table_id = const.SERVICES_CLASSIFICATION_TABLE
 
         parser = self.get_datapath().ofproto_parser
+        ofproto = self.get_datapath().ofproto
 
         # defaults of sg-table to drop packet
         drop_inst = None
@@ -739,9 +739,11 @@ class SGApp(DFlowApp):
              match=match)
 
         # rel state, pass
-        match = parser.OFPMatch(ct_state=(const.CT_STATE_TRK |
-                                          const.CT_STATE_REL,
-                                          SG_CT_STATE_MASK))
+        ct_related_not_new_flag = const.CT_STATE_TRK | const.CT_STATE_REL
+        ct_related_mask = const.CT_STATE_TRK | const.CT_STATE_REL | \
+            const.CT_STATE_NEW | const.CT_STATE_INV
+        match = parser.OFPMatch(ct_state=(ct_related_not_new_flag,
+                                          ct_related_mask))
         self.mod_flow(
              self.get_datapath(),
              inst=goto_inst,
@@ -749,6 +751,28 @@ class SGApp(DFlowApp):
              priority=const.PRIORITY_CT_STATE,
              match=match)
 
+        ct_related_new_flag = const.CT_STATE_TRK | const.CT_STATE_REL | \
+            const.CT_STATE_NEW
+        match = parser.OFPMatch(eth_type=ether.ETH_TYPE_IP,
+                                ct_state=(ct_related_new_flag,
+                                          ct_related_mask))
+        actions = [parser.NXActionCT(actions=[],
+                                     alg=0,
+                                     flags=const.CT_FLAG_COMMIT,
+                                     recirc_table=goto_table_id,
+                                     zone_ofs_nbits=15,
+                                     zone_src=const.CT_ZONE_REG)]
+        action_inst = self.get_datapath(). \
+            ofproto_parser.OFPInstructionActions(
+            ofproto.OFPIT_APPLY_ACTIONS, actions)
+        inst = [action_inst]
+        self.mod_flow(
+             self.get_datapath(),
+             inst=inst,
+             table_id=table_id,
+             priority=const.PRIORITY_CT_STATE,
+             match=match)
+
         # inv state, drop
         invalid_ct_state_flag = const.CT_STATE_TRK | const.CT_STATE_INV
         match = parser.OFPMatch(ct_state=(invalid_ct_state_flag,
@@ -975,7 +999,7 @@ class SGApp(DFlowApp):
             return
 
         secgroups = lport.get_security_groups()
-        if secgroups is None:
+        if not secgroups:
             return
 
         # uninstall ct table
@@ -990,7 +1014,7 @@ class SGApp(DFlowApp):
             return
 
         secgroups = lport.get_security_groups()
-        if secgroups is None:
+        if not secgroups:
             return
 
         for secgroup_id in secgroups:
@@ -1008,7 +1032,7 @@ class SGApp(DFlowApp):
             self._get_added_and_removed_secgroups(secgroups,
                                                   original_secgroups)
 
-        if secgroups is None and original_secgroups:
+        if not secgroups and original_secgroups:
             # uninstall ct table
             self._uninstall_connection_track_flows(lport)
 
@@ -1018,7 +1042,7 @@ class SGApp(DFlowApp):
         for secgroup_id in removed_secgroups:
             self._remove_local_port_associating(lport, secgroup_id)
 
-        if secgroups and original_secgroups is None:
+        if secgroups and not original_secgroups:
             # install ct table
             self._install_connection_track_flows(lport)
 
@@ -1046,7 +1070,7 @@ class SGApp(DFlowApp):
             return
 
         secgroups = lport.get_security_groups()
-        if secgroups is None:
+        if not secgroups:
             return
 
         for secgroup_id in secgroups:
@@ -1061,7 +1085,7 @@ class SGApp(DFlowApp):
             return
 
         secgroups = lport.get_security_groups()
-        if secgroups is None:
+        if not secgroups:
             return
 
         for secgroup_id in secgroups:
