@@ -13,10 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib.db import model_base
 import sqlalchemy as sa
 
-from neutron.db import model_base
-
 
 class DFLockedObjects(model_base.BASEV2):
     __tablename__ = 'dflockedobjects'
@@ -24,7 +23,7 @@ class DFLockedObjects(model_base.BASEV2):
     object_uuid = sa.Column(sa.String(36), primary_key=True)
     lock = sa.Column(sa.Boolean, default=False)
     session_id = sa.Column(sa.BigInteger, default=0)
-    created_at = sa.Column(sa.DateTime)
+    created_at = sa.Column(sa.DateTime, onupdate=sa.func.utc_timestamp())
 
 
 class DFVersionObjects(model_base.BASEV2):
