@@ -11,25 +11,29 @@
 #    under the License.
 
 import collections
-import eventlet
 import fcntl
 import os
-import pytun
 import re
-import ryu.lib.packet
 import socket
 import threading
 import time
 
-from oslo_log import log
-
+import eventlet
 from neutron.agent.common import utils
+from oslo_config import cfg
+from oslo_log import log
+import pytun
+import ryu.lib.packet
 
 from dragonflow._i18n import _LI, _LE
-from dragonflow.common.utils import DFDaemon
+from dragonflow.common import common_params
+from dragonflow.common import utils as d_utils
 from dragonflow.tests.common import utils as test_utils
 from dragonflow.tests.fullstack import test_objects as objects
 
+
+cfg.CONF.register_opts(common_params.df_opts, 'df')
+
 LOG = log.getLogger(__name__)
 
 
@@ -156,7 +160,17 @@ class Subnet(object):
         self.subnet.create(subnet={
             'cidr': cidr,
             'ip_version': 4,
-            'network_id': topology.network.network_id
+            'network_id': topology.network.network_id,
+            'host_routes': [
+                {
+                    'destination': '1.1.1.0/24',
+                    'nexthop': '2.2.2.2'
+                },
+                {
+                    'destination': '1.1.2.0/24',
+                    'nexthop': '3.3.3.3'
+                },
+            ]
         })
 
     def delete(self):
@@ -239,6 +253,7 @@ class LogicalPortTap(object):
         :type port:  Port
         """
         self.port = port
+        self.integration_bridge = cfg.CONF.df.integration_bridge
         self.lport = self.port.get_logical_port()
         self.tap = self._create_tap_device()
         self.is_blocking = True
@@ -248,7 +263,7 @@ class LogicalPortTap(object):
         name = self._get_tap_interface_name()
         create_tap_dev(name, self.lport.get_mac())
         tap = pytun.TunTapDevice(flags=flags, name=name)
-        self._connect_tap_device_to_vswitch('br-int', tap.name)
+        self._connect_tap_device_to_vswitch(self.integration_bridge, tap.name)
         tap.up()
         return tap
 
@@ -276,7 +291,8 @@ class LogicalPortTap(object):
         utils.execute(full_args, run_as_root=True, process_input=None)
 
     def delete(self):
-        self._disconnect_tap_device_to_vswitch('br-int', self.tap.name)
+        self._disconnect_tap_device_to_vswitch(self.integration_bridge,
+                                               self.tap.name)
         LOG.info(_LI('Closing tap interface {} ({})').format(
             self.tap.name,
             self.tap.fileno(),
@@ -433,14 +449,15 @@ class Policy(object):
         :param timeout: After this many seconds, throw an exception
         :type timeout:  Number
         """
+        exception = Exception('Timeout')
         if timeout is not None:
             entry_time = time.time()
         for thread in self.threads:
-            thread.wait(timeout)
+            thread.wait(timeout, exception)
             if timeout is not None:
                 timeout -= time.time() - entry_time
                 if timeout <= 0:
-                    raise Exception('Timeout')
+                    raise exception
 
     def stop(self):
         """Stop all threads. Prepare for a new simulation."""
@@ -727,6 +744,11 @@ class SendAction(Action):
 
 
 class SimulateAndSendAction(SendAction):
+    def __init__(self, subnet_id, port_id, packet):
+        super(SimulateAndSendAction, self).__init__(subnet_id, port_id,
+                                                    packet)
+        self.integration_bridge = cfg.CONF.df.integration_bridge
+
     def _send(self, policy, packet):
         interface_object = self._get_interface_object(policy.topology)
         interface_name = interface_object.tap.name
@@ -735,7 +757,8 @@ class SimulateAndSendAction(SendAction):
         return super(SimulateAndSendAction, self)._send(policy, packet)
 
     def _get_port_number(self, interface_name):
-        ovs_ofctl_args = ['ovs-ofctl', 'dump-ports', 'br-int', interface_name]
+        ovs_ofctl_args = ['ovs-ofctl', 'dump-ports', self.integration_bridge,
+                          interface_name]
         awk_args = ['awk', '/^\\s*port\\s+[0-9]+:/ { print $2 }']
         ofctl_output = utils.execute(
             ovs_ofctl_args,
@@ -756,7 +779,7 @@ class SimulateAndSendAction(SendAction):
         args = [
             'ovs-appctl',
             'ofproto/trace',
-            'br-int',
+            self.integration_bridge,
             'in_port:{}'.format(port_number),
             packet_str,
         ]
@@ -823,7 +846,7 @@ class PortThread(object):
         """
         self.packet_handler = packet_handler
         self.port = port
-        self.daemon = DFDaemon(is_not_light=True)
+        self.daemon = d_utils.DFDaemon(is_not_light=True)
         self.is_working = False
         self.thread_id = None
 
@@ -836,8 +859,8 @@ class PortThread(object):
         if self.thread_id != threading.current_thread().ident:
             self.daemon.stop()
 
-    def wait(self, timeout=None):
-        self.daemon.wait(timeout)
+    def wait(self, timeout=None, exception=None):
+        self.daemon.wait(timeout, exception)
 
     def run(self):
         """Continuously read from the tap device, and send received data to the
