@@ -12,16 +12,12 @@
 
 import contextlib
 
+from neutronclient.common import exceptions as n_exc
 from oslo_concurrency import lockutils
-from oslo_log import log
 
 from dragonflow.tests.common import utils
 from dragonflow.tests.fullstack import test_base
 from dragonflow.tests.fullstack import test_objects as objects
-from neutronclient.common import exceptions as n_exc
-
-
-LOG = log.getLogger(__name__)
 
 
 class TestNeutronAPIandDB(test_base.DFTestBase):
@@ -36,6 +32,18 @@ class TestNeutronAPIandDB(test_base.DFTestBase):
         network.close()
         self.assertFalse(network.exists())
 
+    def test_create_network_with_mtu(self):
+        network = self.store(objects.NetworkTestObj(self.neutron, self.nb_api))
+        network.create()
+        self.assertTrue(network.exists())
+        netobj = network.get_network()
+        lswitch = self.nb_api.get_lswitch(netobj['network']['id'],
+                                          netobj['network']['tenant_id'])
+        net_mtu = lswitch.get_mtu()
+        self.assertEqual(netobj['network']['mtu'], net_mtu)
+        network.close()
+        self.assertFalse(network.exists())
+
     def test_dhcp_port_created(self):
         network = self.store(objects.NetworkTestObj(self.neutron, self.nb_api))
         network_id = network.create()
@@ -82,10 +90,46 @@ class TestNeutronAPIandDB(test_base.DFTestBase):
         self.assertFalse(subnet.exists())
         network.close()
 
+    def test_create_subnet_with_host_routes(self):
+        network = self.store(objects.NetworkTestObj(self.neutron, self.nb_api))
+        network_id = network.create()
+        self.assertTrue(network.exists())
+        subnet = self.store(objects.SubnetTestObj(
+            self.neutron,
+            self.nb_api,
+            network_id,
+        ))
+        subnet_data = {
+            'cidr': '192.168.199.0/24',
+            'ip_version': 4,
+            'network_id': network_id,
+            'host_routes': [
+                {
+                    'destination': '1.1.1.0/24',
+                    'nexthop': '2.2.2.2'
+                },
+                {
+                    'destination': '1.1.2.0/24',
+                    'nexthop': '3.3.3.3'
+                },
+            ]
+        }
+        subnet.create(subnet_data)
+        lswitch = self.nb_api.get_lswitch(network_id)
+        subnet = lswitch.get_subnets()
+        self.assertEqual(
+            0, cmp(subnet_data['host_routes'], subnet[0].get_host_routes())
+        )
+
     def test_create_delete_router(self):
         router = self.store(objects.RouterTestObj(self.neutron, self.nb_api))
-        router.create()
+        router_id = router.create()
         self.assertTrue(router.exists())
+        version1 = self.nb_api.get_router(router_id).get_version()
+        router.update()
+        self.assertTrue(router.exists())
+        version2 = self.nb_api.get_router(router_id).get_version()
+        self.assertTrue(version1 != version2)
         router.close()
         self.assertFalse(router.exists())
 
@@ -175,8 +219,13 @@ class TestNeutronAPIandDB(test_base.DFTestBase):
     def test_create_delete_security_group(self):
         secgroup = self.store(
                         objects.SecGroupTestObj(self.neutron, self.nb_api))
-        secgroup.create()
+        sg_id = secgroup.create()
         self.assertTrue(secgroup.exists())
+        version1 = self.nb_api.get_security_group(sg_id).get_version()
+        secgroup.update()
+        self.assertTrue(secgroup.exists())
+        version2 = self.nb_api.get_security_group(sg_id).get_version()
+        self.assertNotEqual(version1, version2)
         secgroup.close()
         self.assertFalse(secgroup.exists())
 
@@ -440,7 +489,7 @@ class TestNeutronAPIandDB(test_base.DFTestBase):
             objects.PortTestObj(self.neutron, self.nb_api, network_id))
         expected_pairs = [
                 {"ip_address": "192.168.127.201",
-                 "mac_address": "11:22:33:44:55:66"},
+                 "mac_address": "00:22:33:44:55:66"},
                 {"ip_address": "192.168.127.202",
                  "mac_address": "22:22:33:44:55:66"}
         ]
@@ -457,7 +506,7 @@ class TestNeutronAPIandDB(test_base.DFTestBase):
 
         expected_pairs = [
                 {"ip_address": "192.168.127.211",
-                 "mac_address": "33:22:33:44:55:66"},
+                 "mac_address": "00:22:33:44:55:66"},
                 {"ip_address": "192.168.127.212",
                  "mac_address": "44:22:33:44:55:66"}
         ]
